@@ -4,6 +4,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
+#include <glob.h>
 #include <grp.h>
 #include <pwd.h>
 #include <stdio.h>
@@ -17,11 +18,16 @@
 #include "log.h"
 #include "logrotate.h"
 
-static char * tabooExts[] = { ".rpmsave", ".rpmorig", "~" };
-static int tabooCount = sizeof(tabooExts) / sizeof(char *);
+static char * defTabooExts[] = { ".rpmsave", ".rpmorig", "~", ",v" };
+static int defTabooCount = sizeof(defTabooExts) / sizeof(char *);
+
+/* I shouldn't use globals here :-( */
+static char ** tabooExts = NULL;
+int tabooCount = 0;
 
 static int readConfigFile(char * configFile, logInfo * defConfig, 
 			  logInfo ** logsPtr, int * numLogsPtr);
+static int globerr(const char * pathname, int theerr);
 
 static int isolateValue(char * fileName, int lineNum, char * key, 
 			char ** startPtr, char ** endPtr) {
@@ -125,6 +131,12 @@ int readConfigPath(char * path, logInfo * defConfig,
     int here;
     int i;
 
+    if (!tabooExts) {
+	tabooExts = malloc(sizeof(*tabooExts) * defTabooCount);
+	memcpy(tabooExts, defTabooExts, sizeof(*tabooExts) * defTabooCount);
+	tabooCount = defTabooCount;
+    }
+
     if (stat(path, &sb)) {
 	message(MESS_ERROR, "cannot stat %s: %s\n", path, strerror(errno));
 	return 1;
@@ -199,6 +211,14 @@ int readConfigPath(char * path, logInfo * defConfig,
     return 0;
 }
 
+static int globerr(const char * pathname, int theerr) {
+    message(MESS_ERROR, "error accessing %s: %s\n", pathname, 
+            strerror(theerr));
+
+    /* We want the glob operation to continue, so return 0 */
+    return 1;
+}
+
 static int readConfigFile(char * configFile, logInfo * defConfig, 
 			  logInfo ** logsPtr, int * numLogsPtr) {
     int fd;
@@ -207,7 +227,7 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
     int length;
     int lineNum = 1;
     int multiplier;
-    int i;
+    int i, j, k;
     char * scriptStart = NULL;
     char ** scriptDest = NULL;
     logInfo * newlog = defConfig;
@@ -217,10 +237,11 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
     struct passwd * pw;
     int rc;
     char createOwner[200], createGroup[200];
-    mode_t createMode;
+    int createMode;
     struct stat sb, sb2;
+    glob_t globResult;
 
-    /* FIXME: createOWnder and createGroup probably shouldn't be fixed
+    /* FIXME: createOwner and createGroup probably shouldn't be fixed
        length arrays -- of course, if we aren't run setuid in doesn't
        matter much */
 
@@ -319,6 +340,10 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 	    } else if (!strcmp(start, "nocompress")) {
 		newlog->flags &= ~LOG_FLAG_COMPRESS;
 
+		*endtag = oldchar, start = endtag;
+	    } else if (!strcmp(start, "delaycompress")) {
+		newlog->flags |= LOG_FLAG_DELAYCOMPRESS;
+
 		*endtag = oldchar, start = endtag;
 	    } else if (!strcmp(start, "ifempty")) {
 		newlog->flags |= LOG_FLAG_IFEMPTY;
@@ -376,7 +401,6 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 		newlog->flags |= LOG_FLAG_CREATE;
 
 		*endtag = oldchar, start = endtag;
-		while (*start != '\n') start++;
 	    } else if (!strcmp(start, "nocreate")) {
 		newlog->flags &= ~LOG_FLAG_CREATE;
 
@@ -475,6 +499,11 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 						        "mail", &start))) {
 		    return 1;
 		}
+	    } else if (!strcmp(start, "nomail")) {
+		/* hmmm, we could lose memory here, but not much */
+		newlog->logAddress = NULL;
+
+		*endtag = oldchar, start = endtag;
 	    } else if (!strcmp(start, "prerotate")) {
 		*endtag = oldchar, start = endtag;
 
@@ -489,6 +518,47 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 		scriptDest = &newlog->post;
 
 		while (*start != '\n') start++;
+	    } else if (!strcmp(start, "tabooext")) {
+		if (newlog != defConfig) {
+		    message(MESS_ERROR, "%s:%d include may not appear inside "
+			    "of log file definition", configFile, lineNum);
+		    return 1;
+		}
+
+		*endtag = oldchar, start = endtag;
+		if (!isolateValue(configFile, lineNum, "size", &start, 
+				  &endtag)) {
+		    oldchar = *endtag, *endtag = '\0';
+
+		    if (*start == '+') {
+			start++;
+			while (isspace(*start) && *start) start++;
+		    } else {
+			free(tabooExts);
+			tabooCount = 0;
+			tabooExts = malloc(1);
+		    }
+
+		    while (*start) {
+			chptr = start;
+			while (!isspace(*chptr) && *chptr != ',' && *chptr)
+			    chptr++;
+
+			tabooExts = realloc(tabooExts, sizeof(*tabooExts) * 
+						(tabooCount + 1));
+			/* this is a memory leak if the list gets reset */
+			tabooExts[tabooCount] = malloc(chptr - start + 1);
+			strncpy(tabooExts[tabooCount], start, chptr - start);
+			tabooExts[tabooCount][chptr - start] = '\0';
+			tabooCount++;
+
+			start = chptr;
+			if (*start == ',') start++;
+			while (isspace(*start) && *start) start++;
+		    }
+
+		    *endtag = oldchar, start = endtag;
+		}
 	    } else if (!strcmp(start, "include")) {
 		if (newlog != defConfig) {
 		    message(MESS_ERROR, "%s:%d include may not appear inside "
@@ -565,15 +635,38 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 	    oldchar = *endtag;
 	    *endtag = '\0';
 
-	    for (i = 0; i < *numLogsPtr - 1; i++) {
-		if (!strcmp((*logsPtr)[i].fn, start)) {
-		    message(MESS_ERROR, "%s:%d duplicate log entry for %s\n",
-			    configFile, lineNum, start);
-		    return 1;
+	    rc = glob(start, 0, globerr, &globResult);
+	    if (rc == GLOB_ABEND) return 1;
+
+	    newlog->files = malloc(sizeof(*newlog->files) * 
+				   globResult.gl_pathc);
+	    newlog->numFiles = 0;
+
+	    for (i = 0; i < globResult.gl_pathc; i++) {
+		/* if we glob directories we can get false matches */
+		if (lstat(globResult.gl_pathv[i], &sb)) continue;
+		if (S_ISDIR(sb.st_mode)) continue;
+
+		for (j = 0; j < *numLogsPtr - 1; j++) {
+		    for (k = 0; k < (*logsPtr)[j].numFiles; k++) {
+			if (!strcmp((*logsPtr)[j].files[k], 
+				    globResult.gl_pathv[i])) {
+			    message(MESS_ERROR, 
+				    "%s:%d duplicate log entry for %s\n",
+				    configFile, lineNum, 
+				    globResult.gl_pathv[i]);
+			    return 1;
+			}
+		    }
 		}
+
+		newlog->files[newlog->numFiles] = 
+			globResult.gl_pathv[i];
+		newlog->numFiles++;
 	    }
 
-	    newlog->fn = strdup(start);
+	    newlog->globMem = globResult;
+	    newlog->pattern = strdup(start);
 
 	    message(MESS_DEBUG, "reading config info for %s\n", start);
 
@@ -611,22 +704,24 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 		    return 1;
 		}
 
-		dirName = ourDirName(newlog->fn);
-		if (stat(dirName, &sb2)) {
-		    message(MESS_ERROR, "%s:%d error verifying log file "
-				"path %s: %s\n", configFile, lineNum, 
-				dirName, strerror(errno));
+		for (i = 0; i < newlog->numFiles; i++) {
+		    dirName = ourDirName(newlog->files[i]);
+		    if (stat(dirName, &sb2)) {
+			message(MESS_ERROR, "%s:%d error verifying log file "
+				    "path %s: %s\n", configFile, lineNum, 
+				    dirName, strerror(errno));
+			free(dirName);
+			return 1;
+		    }
+
 		    free(dirName);
-		    return 1;
-		}
 
-		free(dirName);
-
-		if (sb.st_dev != sb2.st_dev) {
-		    message(MESS_ERROR, "%s:%d olddir %s and log file %s "
-				"are on different devices\n", configFile,
-				lineNum, newlog->oldDir, newlog->fn);
-		    return 1;
+		    if (sb.st_dev != sb2.st_dev) {
+			message(MESS_ERROR, "%s:%d olddir %s and log file %s "
+				    "are on different devices\n", configFile,
+				    lineNum, newlog->oldDir, newlog->files[i]);
+			return 1;
+		    }
 		}
 	    }
 
